#!/usr/bin/env python
#Read sensor-data from Arduino and publish as laserscan

import rclpy
import std_msgs.msg
from rclpy.node import Node


import serial
import math
import time
import numpy
from geometry_msgs.msg import Vector3
from sensor_msgs.msg import LaserScan, Range


#Initialize arduino communication
arduino = serial.Serial('/dev/ttyACM0', 9600, timeout=.1)

class RangeToLaser(Node):
	def __init__(self, NR_SENSORS, ANGLE_MIN, ANGLE_MAX, RANGE_MIN, RANGE_MAX, START_INDEX, END_INDEX):
		#super().__init__("range_to_laser")
		self.nr_sensor = NR_SENSORS
		self.angle_min = ANGLE_MIN
		self.angle_max = ANGLE_MAX
		self.range_min = RANGE_MIN
		self.range_max = RANGE_MAX
		self.start_index = START_INDEX
		self.end_index = END_INDEX
		timer_period = 0.05
		self.timer = self.create_timer(timer_period, self.scan)
		self.pub_array = []
		self.msg = LaserScan()

		for i in range(self.start_index, self.end_index + 1):
			sensor_num = i + 1
			topic_name = "/distance/range_to_laser%d"%sensor_num
			pub = self.create_publisher(LaserScan, topic_name, 5)
			self.pub_array.append(pub)

		#Laser message
		self.msg.header = std_msgs.msg.Header()
		self.msg.angle_min = math.radians(self.angle_min)
		self.msg.angle_max = math.radians(self.angle_max)
		self.msg.angle_increment = math.radians(1)
		self.msg.time_increment = 0.0
		self.msg.range_min = self.range_min
		self.msg.range_max = self.range_max

	
	def scan(self):
		distance = []
		ranges = []
		self.msg.header.stamp = self.get_clock().now().to_msg()
		data = arduino.readline()
		if data:
			data_string = str(data)
			data_array = data_string.split(',')

		if data and len(data_array) >= self.nr_sensors:
			for i in range(self.start_index, self.end_index + 1):
				dist = data_array[i]
				dist = float(dist)
				if (i == 0):
					dist = float(dist[2:]

				#if (i == (len(data_array) - 1)):
				#	dist = float(dist[:-5]

				#else:
					#dist = float(dist)
				
				distance.append(dist)
					

			
			for i in range(self.nr_sensors):
				ranges = []
				ranges.append(distance[i]*0.01)
				ranges = ranges*20
				header_id = self.start_index + (i+1)
				self.msg.header.frame_id = 'range_%d'%header_id
				self.msg.ranges = ranges
				self.pub_array[i].publish(self.msg)




	



NR_SENSORS, ANGLE_MIN, ANGLE_MAX, RANGE_MIN, RANGE_MAX, START_INDEX, END_INDEX

def main(args=None):
	time.sleep(1)
	rclpy.init(args=args)
	range_to_laser = RangeToLaser(3, -15, 15, 0.05, 4, 1, 3)

	rclpy.spin(range_to_laser)

	range_to_laser.destroy_node()

	rclpy.shutdown()
	

if __name__=="__main__":
	main()

